import { jsx as _jsx } from "react/jsx-runtime";
import { useEffect, useLayoutEffect, useRef } from 'react';
import { useAnchoredCommentsContext } from '../contexts/AnchoredCommentsContext';
const CommentView = ({ commentId, children }) => {
    var _a, _b;
    const { state, dispatch, commentPositions } = useAnchoredCommentsContext();
    const { commentSectionOffsetY, activeCommentId } = state;
    const viewRef = useRef(null);
    // State to track whether the initial positioning has occurred
    const hasPositionedRef = useRef(false);
    const updateHeightAndRecalculate = () => {
        if (!viewRef.current)
            return;
        const height = viewRef.current.getBoundingClientRect().height;
        dispatch({
            type: 'UPDATE_COMMENT_HEIGHT',
            payload: { id: commentId, height },
        });
    };
    // Measure initial height
    useLayoutEffect(() => {
        updateHeightAndRecalculate();
    }, []);
    // Monitor dynamic height changes
    useLayoutEffect(() => {
        if (!viewRef.current)
            return;
        const resizeObserver = new ResizeObserver(() => {
            updateHeightAndRecalculate();
        });
        resizeObserver.observe(viewRef.current);
        return () => {
            resizeObserver.disconnect();
        };
    }, []);
    useEffect(() => {
        if (!viewRef.current)
            return;
        const element = viewRef.current;
        const handleClick = (e) => {
            const target = e.target;
            // Don't focus the parent container if user clicked on:
            // - Buttons: Have their own focus and click behavior
            // - Links: Need to handle navigation
            // - Inputs/Textareas: Need to handle text entry
            // - Custom buttons: Elements with role="button" (like div styled as button)
            const interactiveTags = ['BUTTON', 'A', 'INPUT', 'TEXTAREA'];
            if (interactiveTags.includes(target.tagName) ||
                // closest() checks if the clicked element or any of its parents have role="button"
                // This catches custom button implementations that don't use <button> tags
                target.closest('[role="button"]'))
                return;
            element.focus();
        };
        element.addEventListener('click', handleClick);
        return () => {
            element.removeEventListener('click', handleClick);
        };
    }, []);
    const onDeleteSuccess = () => {
        dispatch({ type: 'DELETE_ANCHOR', payload: { id: commentId } });
    };
    const onFocus = () => {
        dispatch({ type: 'SET_ACTIVE_COMMENT_ID', payload: commentId });
    };
    // Position off-screen when position is unknown
    const adjustedTop = (_b = (_a = commentPositions[commentId]) === null || _a === void 0 ? void 0 : _a.top) !== null && _b !== void 0 ? _b : -9999;
    const isOffScreen = adjustedTop === -9999;
    useEffect(() => {
        if (!isOffScreen && !hasPositionedRef.current)
            setTimeout(() => {
                hasPositionedRef.current = true;
            }, 0); // Force a reflow to trigger transition
    }, [isOffScreen]);
    const shouldTransition = hasPositionedRef.current;
    return (_jsx("div", { ref: viewRef, tabIndex: isOffScreen ? -1 : 0, style: {
            position: 'absolute',
            top: `${adjustedTop - commentSectionOffsetY}px`,
            transition: shouldTransition ? 'top 0.3s ease-out' : 'none',
            left: 0,
            width: '100%',
            pointerEvents: isOffScreen ? 'none' : 'auto',
        }, onFocus: onFocus, children: typeof children === 'function'
            ? children({
                isActive: activeCommentId === commentId,
                onDeleteSuccess,
            })
            : children }));
};
export default CommentView;
//# sourceMappingURL=CommentView.js.map