import React, { ReactNode } from 'react';
import type { CommentAnchor } from '../types';
import { PositionedSelectionRange, Positions } from '../types';
export type AnchoredState = {
    contentSectionOffsetY: number;
    commentSectionOffsetY: number;
    activeCommentId: string | null;
    textPositions: Positions;
    commentHeights: Record<string, number>;
    selection: PositionedSelectionRange | null;
    newComment: {
        id: string;
        selectionRange: PositionedSelectionRange;
        height?: number;
    } | null;
    anchors: CommentAnchor[];
};
export type AnchoredAction = {
    type: 'SET_ACTIVE_COMMENT_ID';
    payload: string | null;
} | {
    type: 'SET_SELECTION';
    payload: PositionedSelectionRange | null;
} | {
    type: 'SET_ACTIVE_COMMENT_AND_SELECTION';
    payload: {
        activeCommentId: string | null;
        selection: PositionedSelectionRange | null;
    };
} | {
    type: 'SHOW_NEW_COMMENT';
} | {
    type: 'CANCEL_NEW_COMMENT';
} | {
    type: 'UPDATE_COMMENT_HEIGHT';
    payload: {
        id: string;
        height: number;
    };
} | {
    type: 'UPDATE_TEXT_POSITIONS';
    payload: Record<string, {
        top: number;
    }>;
} | {
    type: 'UPDATE_CONTENT_SECTION_OFFSETY';
    payload: number;
} | {
    type: 'UPDATE_COMMENT_SECTION_OFFSETY';
    payload: number;
} | {
    type: 'ADD_ANCHOR';
    payload: CommentAnchor;
} | {
    type: 'DELETE_ANCHOR';
    payload: {
        id: string;
    };
} | {
    type: 'SET_ANCHORS';
    payload: CommentAnchor[];
};
type AnchoredCommentsContextType = {
    state: AnchoredState;
    dispatch: React.Dispatch<AnchoredAction>;
    contentViews: React.MutableRefObject<Partial<Record<string, React.RefObject<HTMLDivElement>>>>;
    commentPositions: Positions;
};
export declare const AnchoredCommentsProvider: ({ anchors, children, }: {
    anchors: CommentAnchor[];
    children: ReactNode;
}) => import("react/jsx-runtime").JSX.Element;
export declare const useAnchoredCommentsContext: () => AnchoredCommentsContextType;
export {};
