import { jsx as _jsx } from "react/jsx-runtime";
import { createContext, useContext, useEffect, useReducer, useRef, useState, } from 'react';
import { anchoredReducers } from '../reducers/anchoredReducers';
import { calculatePositions } from '../utils/calculatePositions';
const AnchoredCommentsContext = createContext(null);
export const AnchoredCommentsProvider = ({ anchors, children, }) => {
    const [state, dispatch] = useReducer(anchoredReducers, {
        contentSectionOffsetY: 0,
        commentSectionOffsetY: 0,
        activeCommentId: null,
        textPositions: {},
        commentHeights: {},
        selection: null,
        newComment: null,
        anchors,
    });
    const contentViews = useRef({});
    const [commentPositions, setCommentPositions] = useState({});
    useEffect(() => {
        dispatch({ type: 'SET_ANCHORS', payload: anchors });
    }, [
        anchors
            .map(a => a.id)
            .sort((a, b) => a.localeCompare(b))
            .join(','),
    ]);
    useEffect(() => {
        const textPositionsWithNewComment = Object.assign(Object.assign({}, state.textPositions), (state.newComment && {
            [state.newComment.id]: {
                top: state.newComment.selectionRange.positionTop,
            },
        }));
        const visibleComments = new Set([
            ...state.anchors.map(comment => comment.id),
            ...(state.newComment ? [state.newComment.id] : []),
        ]);
        const positions = calculatePositions(textPositionsWithNewComment, state.activeCommentId, visibleComments, state.commentHeights);
        setCommentPositions(positions);
    }, [
        state.textPositions,
        state.newComment,
        state.anchors,
        state.activeCommentId,
        state.commentHeights,
    ]);
    return (_jsx(AnchoredCommentsContext.Provider, { value: {
            state,
            dispatch,
            contentViews,
            commentPositions,
        }, children: children }));
};
export const useAnchoredCommentsContext = () => {
    const context = useContext(AnchoredCommentsContext);
    if (!context) {
        throw new Error('useAnchoredCommentsContext must be used within a AnchoredCommentsProvider');
    }
    return context;
};
//# sourceMappingURL=AnchoredCommentsContext.js.map